from . import statistics

def calculate_array_histogram(data_container_array, selected_array_path = None, number_of_bins = None, min_range = None, max_range = None, user_defined_range = None, normalize = None, new_attribute_matrix_name = None, new_data_array_name = None, new_data_container = None, new_data_container_name = None, observer = None) -> int:
  """
  CalculateArrayHistogram

  :param data_container_array | DataContainerArray
  :param selected_array_path | DataArrayPath SelectedArrayPath
  :param number_of_bins | int NumberOfBins
  :param min_range | double MinRange
  :param max_range | double MaxRange
  :param user_defined_range | bool UserDefinedRange
  :param normalize | bool Normalize
  :param new_attribute_matrix_name | QString NewAttributeMatrixName
  :param new_data_array_name | QString NewDataArrayName
  :param new_data_container | bool NewDataContainer
  :param new_data_container_name | DataArrayPath NewDataContainerName
  :param observer | Observer
  """

  filter = statistics.CalculateArrayHistogram()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_array_path is not None:
    filter.SelectedArrayPath = selected_array_path
  if number_of_bins is not None:
    filter.NumberOfBins = number_of_bins
  if min_range is not None:
    filter.MinRange = min_range
  if max_range is not None:
    filter.MaxRange = max_range
  if user_defined_range is not None:
    filter.UserDefinedRange = user_defined_range
  if normalize is not None:
    filter.Normalize = normalize
  if new_attribute_matrix_name is not None:
    filter.NewAttributeMatrixName = new_attribute_matrix_name
  if new_data_array_name is not None:
    filter.NewDataArrayName = new_data_array_name
  if new_data_container is not None:
    filter.NewDataContainer = new_data_container
  if new_data_container_name is not None:
    filter.NewDataContainerName = new_data_container_name
  filter.execute()
  return filter.ErrorCode

def compute_moment_invariants2_d(data_container_array, feature_ids_array_path = None, feature_rect_array_path = None, normalize_moment_invariants = None, omega1_array_path = None, omega2_array_path = None, save_central_moments = None, central_moments_array_path = None, observer = None) -> int:
  """
  ComputeMomentInvariants2D

  :param data_container_array | DataContainerArray
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param feature_rect_array_path | DataArrayPath FeatureRectArrayPath
  :param normalize_moment_invariants | bool NormalizeMomentInvariants
  :param omega1_array_path | DataArrayPath Omega1ArrayPath
  :param omega2_array_path | DataArrayPath Omega2ArrayPath
  :param save_central_moments | bool SaveCentralMoments
  :param central_moments_array_path | DataArrayPath CentralMomentsArrayPath
  :param observer | Observer
  """

  filter = statistics.ComputeMomentInvariants2D()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  if feature_rect_array_path is not None:
    filter.FeatureRectArrayPath = feature_rect_array_path
  if normalize_moment_invariants is not None:
    filter.NormalizeMomentInvariants = normalize_moment_invariants
  if omega1_array_path is not None:
    filter.Omega1ArrayPath = omega1_array_path
  if omega2_array_path is not None:
    filter.Omega2ArrayPath = omega2_array_path
  if save_central_moments is not None:
    filter.SaveCentralMoments = save_central_moments
  if central_moments_array_path is not None:
    filter.CentralMomentsArrayPath = central_moments_array_path
  filter.execute()
  return filter.ErrorCode

def find_avg_scalar_value_for_features(data_container_array, selected_cell_array_path = None, feature_ids_array_path = None, new_feature_array_array_path = None, observer = None) -> int:
  """
  FindAvgScalarValueForFeatures

  :param data_container_array | DataContainerArray
  :param selected_cell_array_path | DataArrayPath SelectedCellArrayPath
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param new_feature_array_array_path | DataArrayPath NewFeatureArrayArrayPath
  :param observer | Observer
  """

  filter = statistics.FindAvgScalarValueForFeatures()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_cell_array_path is not None:
    filter.SelectedCellArrayPath = selected_cell_array_path
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  if new_feature_array_array_path is not None:
    filter.NewFeatureArrayArrayPath = new_feature_array_array_path
  filter.execute()
  return filter.ErrorCode

def find_boundary_element_fractions(data_container_array, feature_ids_array_path = None, boundary_cells_array_path = None, boundary_cell_fractions_array_path = None, observer = None) -> int:
  """
  FindBoundaryElementFractions

  :param data_container_array | DataContainerArray
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param boundary_cells_array_path | DataArrayPath BoundaryCellsArrayPath
  :param boundary_cell_fractions_array_path | DataArrayPath BoundaryCellFractionsArrayPath
  :param observer | Observer
  """

  filter = statistics.FindBoundaryElementFractions()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  if boundary_cells_array_path is not None:
    filter.BoundaryCellsArrayPath = boundary_cells_array_path
  if boundary_cell_fractions_array_path is not None:
    filter.BoundaryCellFractionsArrayPath = boundary_cell_fractions_array_path
  filter.execute()
  return filter.ErrorCode

def find_difference_map(data_container_array, first_input_array_path = None, second_input_array_path = None, difference_map_array_path = None, observer = None) -> int:
  """
  FindDifferenceMap

  :param data_container_array | DataContainerArray
  :param first_input_array_path | DataArrayPath FirstInputArrayPath
  :param second_input_array_path | DataArrayPath SecondInputArrayPath
  :param difference_map_array_path | DataArrayPath DifferenceMapArrayPath
  :param observer | Observer
  """

  filter = statistics.FindDifferenceMap()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if first_input_array_path is not None:
    filter.FirstInputArrayPath = first_input_array_path
  if second_input_array_path is not None:
    filter.SecondInputArrayPath = second_input_array_path
  if difference_map_array_path is not None:
    filter.DifferenceMapArrayPath = difference_map_array_path
  filter.execute()
  return filter.ErrorCode

def find_euclidean_dist_map(data_container_array, feature_ids_array_path = None, gb_distances_array_name = None, tj_distances_array_name = None, qp_distances_array_name = None, nearest_neighbors_array_name = None, do_boundaries = None, do_triple_lines = None, do_quad_points = None, save_nearest_neighbors = None, calc_manhattan_dist = None, observer = None) -> int:
  """
  FindEuclideanDistMap

  :param data_container_array | DataContainerArray
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param gb_distances_array_name | QString GBDistancesArrayName
  :param tj_distances_array_name | QString TJDistancesArrayName
  :param qp_distances_array_name | QString QPDistancesArrayName
  :param nearest_neighbors_array_name | QString NearestNeighborsArrayName
  :param do_boundaries | bool DoBoundaries
  :param do_triple_lines | bool DoTripleLines
  :param do_quad_points | bool DoQuadPoints
  :param save_nearest_neighbors | bool SaveNearestNeighbors
  :param calc_manhattan_dist | bool CalcManhattanDist
  :param observer | Observer
  """

  filter = statistics.FindEuclideanDistMap()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  if gb_distances_array_name is not None:
    filter.GBDistancesArrayName = gb_distances_array_name
  if tj_distances_array_name is not None:
    filter.TJDistancesArrayName = tj_distances_array_name
  if qp_distances_array_name is not None:
    filter.QPDistancesArrayName = qp_distances_array_name
  if nearest_neighbors_array_name is not None:
    filter.NearestNeighborsArrayName = nearest_neighbors_array_name
  if do_boundaries is not None:
    filter.DoBoundaries = do_boundaries
  if do_triple_lines is not None:
    filter.DoTripleLines = do_triple_lines
  if do_quad_points is not None:
    filter.DoQuadPoints = do_quad_points
  if save_nearest_neighbors is not None:
    filter.SaveNearestNeighbors = save_nearest_neighbors
  if calc_manhattan_dist is not None:
    filter.CalcManhattanDist = calc_manhattan_dist
  filter.execute()
  return filter.ErrorCode

def find_feature_clustering(data_container_array, number_of_bins = None, phase_number = None, cell_ensemble_attribute_matrix_name = None, remove_biased_features = None, biased_features_array_path = None, equivalent_diameters_array_path = None, feature_phases_array_path = None, centroids_array_path = None, clustering_list_array_name = None, new_ensemble_array_array_name = None, max_min_array_name = None, observer = None) -> int:
  """
  FindFeatureClustering

  :param data_container_array | DataContainerArray
  :param number_of_bins | int NumberOfBins
  :param phase_number | int PhaseNumber
  :param cell_ensemble_attribute_matrix_name | DataArrayPath CellEnsembleAttributeMatrixName
  :param remove_biased_features | bool RemoveBiasedFeatures
  :param biased_features_array_path | DataArrayPath BiasedFeaturesArrayPath
  :param equivalent_diameters_array_path | DataArrayPath EquivalentDiametersArrayPath
  :param feature_phases_array_path | DataArrayPath FeaturePhasesArrayPath
  :param centroids_array_path | DataArrayPath CentroidsArrayPath
  :param clustering_list_array_name | QString ClusteringListArrayName
  :param new_ensemble_array_array_name | QString NewEnsembleArrayArrayName
  :param max_min_array_name | QString MaxMinArrayName
  :param observer | Observer
  """

  filter = statistics.FindFeatureClustering()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if number_of_bins is not None:
    filter.NumberOfBins = number_of_bins
  if phase_number is not None:
    filter.PhaseNumber = phase_number
  if cell_ensemble_attribute_matrix_name is not None:
    filter.CellEnsembleAttributeMatrixName = cell_ensemble_attribute_matrix_name
  if remove_biased_features is not None:
    filter.RemoveBiasedFeatures = remove_biased_features
  if biased_features_array_path is not None:
    filter.BiasedFeaturesArrayPath = biased_features_array_path
  if equivalent_diameters_array_path is not None:
    filter.EquivalentDiametersArrayPath = equivalent_diameters_array_path
  if feature_phases_array_path is not None:
    filter.FeaturePhasesArrayPath = feature_phases_array_path
  if centroids_array_path is not None:
    filter.CentroidsArrayPath = centroids_array_path
  if clustering_list_array_name is not None:
    filter.ClusteringListArrayName = clustering_list_array_name
  if new_ensemble_array_array_name is not None:
    filter.NewEnsembleArrayArrayName = new_ensemble_array_array_name
  if max_min_array_name is not None:
    filter.MaxMinArrayName = max_min_array_name
  filter.execute()
  return filter.ErrorCode

def find_largest_cross_sections(data_container_array, int = None, feature_ids_array_path = None, largest_cross_sections_array_path = None, observer = None) -> int:
  """
  FindLargestCrossSections

  :param data_container_array | DataContainerArray
  :param int | unsigned int
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param largest_cross_sections_array_path | DataArrayPath LargestCrossSectionsArrayPath
  :param observer | Observer
  """

  filter = statistics.FindLargestCrossSections()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if int is not None:
    filter.int = int
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  if largest_cross_sections_array_path is not None:
    filter.LargestCrossSectionsArrayPath = largest_cross_sections_array_path
  filter.execute()
  return filter.ErrorCode

def find_neighborhoods(data_container_array, neighborhood_list_array_name = None, multiples_of_average = None, equivalent_diameters_array_path = None, feature_phases_array_path = None, centroids_array_path = None, neighborhoods_array_name = None, observer = None) -> int:
  """
  FindNeighborhoods

  :param data_container_array | DataContainerArray
  :param neighborhood_list_array_name | QString NeighborhoodListArrayName
  :param multiples_of_average | float MultiplesOfAverage
  :param equivalent_diameters_array_path | DataArrayPath EquivalentDiametersArrayPath
  :param feature_phases_array_path | DataArrayPath FeaturePhasesArrayPath
  :param centroids_array_path | DataArrayPath CentroidsArrayPath
  :param neighborhoods_array_name | QString NeighborhoodsArrayName
  :param observer | Observer
  """

  filter = statistics.FindNeighborhoods()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if neighborhood_list_array_name is not None:
    filter.NeighborhoodListArrayName = neighborhood_list_array_name
  if multiples_of_average is not None:
    filter.MultiplesOfAverage = multiples_of_average
  if equivalent_diameters_array_path is not None:
    filter.EquivalentDiametersArrayPath = equivalent_diameters_array_path
  if feature_phases_array_path is not None:
    filter.FeaturePhasesArrayPath = feature_phases_array_path
  if centroids_array_path is not None:
    filter.CentroidsArrayPath = centroids_array_path
  if neighborhoods_array_name is not None:
    filter.NeighborhoodsArrayName = neighborhoods_array_name
  filter.execute()
  return filter.ErrorCode

def find_neighbors(data_container_array, cell_feature_attribute_matrix_path = None, shared_surface_area_list_array_name = None, neighbor_list_array_name = None, feature_ids_array_path = None, boundary_cells_array_name = None, num_neighbors_array_name = None, surface_features_array_name = None, store_boundary_cells = None, store_surface_features = None, observer = None) -> int:
  """
  FindNeighbors

  :param data_container_array | DataContainerArray
  :param cell_feature_attribute_matrix_path | DataArrayPath CellFeatureAttributeMatrixPath
  :param shared_surface_area_list_array_name | QString SharedSurfaceAreaListArrayName
  :param neighbor_list_array_name | QString NeighborListArrayName
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param boundary_cells_array_name | QString BoundaryCellsArrayName
  :param num_neighbors_array_name | QString NumNeighborsArrayName
  :param surface_features_array_name | QString SurfaceFeaturesArrayName
  :param store_boundary_cells | bool StoreBoundaryCells
  :param store_surface_features | bool StoreSurfaceFeatures
  :param observer | Observer
  """

  filter = statistics.FindNeighbors()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if cell_feature_attribute_matrix_path is not None:
    filter.CellFeatureAttributeMatrixPath = cell_feature_attribute_matrix_path
  if shared_surface_area_list_array_name is not None:
    filter.SharedSurfaceAreaListArrayName = shared_surface_area_list_array_name
  if neighbor_list_array_name is not None:
    filter.NeighborListArrayName = neighbor_list_array_name
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  if boundary_cells_array_name is not None:
    filter.BoundaryCellsArrayName = boundary_cells_array_name
  if num_neighbors_array_name is not None:
    filter.NumNeighborsArrayName = num_neighbors_array_name
  if surface_features_array_name is not None:
    filter.SurfaceFeaturesArrayName = surface_features_array_name
  if store_boundary_cells is not None:
    filter.StoreBoundaryCells = store_boundary_cells
  if store_surface_features is not None:
    filter.StoreSurfaceFeatures = store_surface_features
  filter.execute()
  return filter.ErrorCode

def find_num_features(data_container_array, feature_phases_array_path = None, num_features_array_path = None, observer = None) -> int:
  """
  FindNumFeatures

  :param data_container_array | DataContainerArray
  :param feature_phases_array_path | DataArrayPath FeaturePhasesArrayPath
  :param num_features_array_path | DataArrayPath NumFeaturesArrayPath
  :param observer | Observer
  """

  filter = statistics.FindNumFeatures()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if feature_phases_array_path is not None:
    filter.FeaturePhasesArrayPath = feature_phases_array_path
  if num_features_array_path is not None:
    filter.NumFeaturesArrayPath = num_features_array_path
  filter.execute()
  return filter.ErrorCode

def find_shapes(data_container_array, cell_feature_attribute_matrix_name = None, feature_ids_array_path = None, centroids_array_path = None, omega3s_array_name = None, volumes_array_name = None, axis_lengths_array_name = None, axis_euler_angles_array_name = None, aspect_ratios_array_name = None, observer = None) -> int:
  """
  FindShapes

  :param data_container_array | DataContainerArray
  :param cell_feature_attribute_matrix_name | DataArrayPath CellFeatureAttributeMatrixName
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param centroids_array_path | DataArrayPath CentroidsArrayPath
  :param omega3s_array_name | QString Omega3sArrayName
  :param volumes_array_name | QString VolumesArrayName
  :param axis_lengths_array_name | QString AxisLengthsArrayName
  :param axis_euler_angles_array_name | QString AxisEulerAnglesArrayName
  :param aspect_ratios_array_name | QString AspectRatiosArrayName
  :param observer | Observer
  """

  filter = statistics.FindShapes()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if cell_feature_attribute_matrix_name is not None:
    filter.CellFeatureAttributeMatrixName = cell_feature_attribute_matrix_name
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  if centroids_array_path is not None:
    filter.CentroidsArrayPath = centroids_array_path
  if omega3s_array_name is not None:
    filter.Omega3sArrayName = omega3s_array_name
  if volumes_array_name is not None:
    filter.VolumesArrayName = volumes_array_name
  if axis_lengths_array_name is not None:
    filter.AxisLengthsArrayName = axis_lengths_array_name
  if axis_euler_angles_array_name is not None:
    filter.AxisEulerAnglesArrayName = axis_euler_angles_array_name
  if aspect_ratios_array_name is not None:
    filter.AspectRatiosArrayName = aspect_ratios_array_name
  filter.execute()
  return filter.ErrorCode

def find_sizes(data_container_array, feature_attribute_matrix_name = None, feature_ids_array_path = None, volumes_array_name = None, equivalent_diameters_array_name = None, num_elements_array_name = None, save_element_sizes = None, observer = None) -> int:
  """
  FindSizes

  :param data_container_array | DataContainerArray
  :param feature_attribute_matrix_name | DataArrayPath FeatureAttributeMatrixName
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param volumes_array_name | QString VolumesArrayName
  :param equivalent_diameters_array_name | QString EquivalentDiametersArrayName
  :param num_elements_array_name | QString NumElementsArrayName
  :param save_element_sizes | bool SaveElementSizes
  :param observer | Observer
  """

  filter = statistics.FindSizes()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if feature_attribute_matrix_name is not None:
    filter.FeatureAttributeMatrixName = feature_attribute_matrix_name
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  if volumes_array_name is not None:
    filter.VolumesArrayName = volumes_array_name
  if equivalent_diameters_array_name is not None:
    filter.EquivalentDiametersArrayName = equivalent_diameters_array_name
  if num_elements_array_name is not None:
    filter.NumElementsArrayName = num_elements_array_name
  if save_element_sizes is not None:
    filter.SaveElementSizes = save_element_sizes
  filter.execute()
  return filter.ErrorCode

def find_surface_area_to_volume(data_container_array, feature_ids_array_path = None, num_cells_array_path = None, surface_area_volume_ratio_array_name = None, sphericity_array_name = None, calculate_sphericity = None, observer = None) -> int:
  """
  FindSurfaceAreaToVolume

  :param data_container_array | DataContainerArray
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param num_cells_array_path | DataArrayPath NumCellsArrayPath
  :param surface_area_volume_ratio_array_name | QString SurfaceAreaVolumeRatioArrayName
  :param sphericity_array_name | QString SphericityArrayName
  :param calculate_sphericity | bool CalculateSphericity
  :param observer | Observer
  """

  filter = statistics.FindSurfaceAreaToVolume()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  if num_cells_array_path is not None:
    filter.NumCellsArrayPath = num_cells_array_path
  if surface_area_volume_ratio_array_name is not None:
    filter.SurfaceAreaVolumeRatioArrayName = surface_area_volume_ratio_array_name
  if sphericity_array_name is not None:
    filter.SphericityArrayName = sphericity_array_name
  if calculate_sphericity is not None:
    filter.CalculateSphericity = calculate_sphericity
  filter.execute()
  return filter.ErrorCode

def find_vol_fractions(data_container_array, cell_phases_array_path = None, vol_fractions_array_path = None, observer = None) -> int:
  """
  FindVolFractions

  :param data_container_array | DataContainerArray
  :param cell_phases_array_path | DataArrayPath CellPhasesArrayPath
  :param vol_fractions_array_path | DataArrayPath VolFractionsArrayPath
  :param observer | Observer
  """

  filter = statistics.FindVolFractions()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if cell_phases_array_path is not None:
    filter.CellPhasesArrayPath = cell_phases_array_path
  if vol_fractions_array_path is not None:
    filter.VolFractionsArrayPath = vol_fractions_array_path
  filter.execute()
  return filter.ErrorCode

def fit_feature_data(data_container_array, selected_feature_array_path = None, int = None, remove_biased_features = None, feature_phases_array_path = None, biased_features_array_path = None, new_ensemble_array_array = None, observer = None) -> int:
  """
  FitFeatureData

  :param data_container_array | DataContainerArray
  :param selected_feature_array_path | DataArrayPath SelectedFeatureArrayPath
  :param int | unsigned int
  :param remove_biased_features | bool RemoveBiasedFeatures
  :param feature_phases_array_path | DataArrayPath FeaturePhasesArrayPath
  :param biased_features_array_path | DataArrayPath BiasedFeaturesArrayPath
  :param new_ensemble_array_array | DataArrayPath NewEnsembleArrayArray
  :param observer | Observer
  """

  filter = statistics.FitFeatureData()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_feature_array_path is not None:
    filter.SelectedFeatureArrayPath = selected_feature_array_path
  if int is not None:
    filter.int = int
  if remove_biased_features is not None:
    filter.RemoveBiasedFeatures = remove_biased_features
  if feature_phases_array_path is not None:
    filter.FeaturePhasesArrayPath = feature_phases_array_path
  if biased_features_array_path is not None:
    filter.BiasedFeaturesArrayPath = biased_features_array_path
  if new_ensemble_array_array is not None:
    filter.NewEnsembleArrayArray = new_ensemble_array_array
  filter.execute()
  return filter.ErrorCode

def generate_ensemble_statistics(data_container_array, cell_ensemble_attribute_matrix_path = None, phase_types_array_name = None, phase_type_array = None, phase_type_data = None, neighbor_list_array_path = None, shared_surface_area_list_array_path = None, feature_phases_array_path = None, biased_features_array_path = None, equivalent_diameters_array_path = None, neighborhoods_array_path = None, aspect_ratios_array_path = None, omega3s_array_path = None, axis_euler_angles_array_path = None, crystal_structures_array_path = None, surface_features_array_path = None, volumes_array_path = None, rdf_array_path = None, max_min_rdf_array_path = None, feature_euler_angles_array_path = None, avg_quats_array_path = None, statistics_array_name = None, include_radial_dist_func = None, calculate_morphological_stats = None, size_distribution_fit_type = None, compute_aspect_ratio_distribution = None, aspect_ratio_distribution_fit_type = None, compute_omega3_distribution = None, omega3_distribution_fit_type = None, compute_neighborhood_distribution = None, neighborhood_distribution_fit_type = None, calculate_crystallographic_stats = None, calculate_odf = None, calculate_mdf = None, calculate_axis_odf = None, size_correlation_resolution = None, observer = None) -> int:
  """
  GenerateEnsembleStatistics

  :param data_container_array | DataContainerArray
  :param cell_ensemble_attribute_matrix_path | DataArrayPath CellEnsembleAttributeMatrixPath
  :param phase_types_array_name | QString PhaseTypesArrayName
  :param phase_type_array | PhaseType::Types PhaseTypeArray
  :param phase_type_data | PhaseType::Types PhaseTypeData
  :param neighbor_list_array_path | DataArrayPath NeighborListArrayPath
  :param shared_surface_area_list_array_path | DataArrayPath SharedSurfaceAreaListArrayPath
  :param feature_phases_array_path | DataArrayPath FeaturePhasesArrayPath
  :param biased_features_array_path | DataArrayPath BiasedFeaturesArrayPath
  :param equivalent_diameters_array_path | DataArrayPath EquivalentDiametersArrayPath
  :param neighborhoods_array_path | DataArrayPath NeighborhoodsArrayPath
  :param aspect_ratios_array_path | DataArrayPath AspectRatiosArrayPath
  :param omega3s_array_path | DataArrayPath Omega3sArrayPath
  :param axis_euler_angles_array_path | DataArrayPath AxisEulerAnglesArrayPath
  :param crystal_structures_array_path | DataArrayPath CrystalStructuresArrayPath
  :param surface_features_array_path | DataArrayPath SurfaceFeaturesArrayPath
  :param volumes_array_path | DataArrayPath VolumesArrayPath
  :param rdf_array_path | DataArrayPath RDFArrayPath
  :param max_min_rdf_array_path | DataArrayPath MaxMinRDFArrayPath
  :param feature_euler_angles_array_path | DataArrayPath FeatureEulerAnglesArrayPath
  :param avg_quats_array_path | DataArrayPath AvgQuatsArrayPath
  :param statistics_array_name | QString StatisticsArrayName
  :param include_radial_dist_func | bool IncludeRadialDistFunc
  :param calculate_morphological_stats | bool CalculateMorphologicalStats
  :param size_distribution_fit_type | int SizeDistributionFitType
  :param compute_aspect_ratio_distribution | bool ComputeAspectRatioDistribution
  :param aspect_ratio_distribution_fit_type | int AspectRatioDistributionFitType
  :param compute_omega3_distribution | bool ComputeOmega3Distribution
  :param omega3_distribution_fit_type | int Omega3DistributionFitType
  :param compute_neighborhood_distribution | bool ComputeNeighborhoodDistribution
  :param neighborhood_distribution_fit_type | int NeighborhoodDistributionFitType
  :param calculate_crystallographic_stats | bool CalculateCrystallographicStats
  :param calculate_odf | bool CalculateODF
  :param calculate_mdf | bool CalculateMDF
  :param calculate_axis_odf | bool CalculateAxisODF
  :param size_correlation_resolution | float SizeCorrelationResolution
  :param observer | Observer
  """

  filter = statistics.GenerateEnsembleStatistics()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if cell_ensemble_attribute_matrix_path is not None:
    filter.CellEnsembleAttributeMatrixPath = cell_ensemble_attribute_matrix_path
  if phase_types_array_name is not None:
    filter.PhaseTypesArrayName = phase_types_array_name
  if phase_type_array is not None:
    filter.PhaseTypeArray = phase_type_array
  if phase_type_data is not None:
    filter.PhaseTypeData = phase_type_data
  if neighbor_list_array_path is not None:
    filter.NeighborListArrayPath = neighbor_list_array_path
  if shared_surface_area_list_array_path is not None:
    filter.SharedSurfaceAreaListArrayPath = shared_surface_area_list_array_path
  if feature_phases_array_path is not None:
    filter.FeaturePhasesArrayPath = feature_phases_array_path
  if biased_features_array_path is not None:
    filter.BiasedFeaturesArrayPath = biased_features_array_path
  if equivalent_diameters_array_path is not None:
    filter.EquivalentDiametersArrayPath = equivalent_diameters_array_path
  if neighborhoods_array_path is not None:
    filter.NeighborhoodsArrayPath = neighborhoods_array_path
  if aspect_ratios_array_path is not None:
    filter.AspectRatiosArrayPath = aspect_ratios_array_path
  if omega3s_array_path is not None:
    filter.Omega3sArrayPath = omega3s_array_path
  if axis_euler_angles_array_path is not None:
    filter.AxisEulerAnglesArrayPath = axis_euler_angles_array_path
  if crystal_structures_array_path is not None:
    filter.CrystalStructuresArrayPath = crystal_structures_array_path
  if surface_features_array_path is not None:
    filter.SurfaceFeaturesArrayPath = surface_features_array_path
  if volumes_array_path is not None:
    filter.VolumesArrayPath = volumes_array_path
  if rdf_array_path is not None:
    filter.RDFArrayPath = rdf_array_path
  if max_min_rdf_array_path is not None:
    filter.MaxMinRDFArrayPath = max_min_rdf_array_path
  if feature_euler_angles_array_path is not None:
    filter.FeatureEulerAnglesArrayPath = feature_euler_angles_array_path
  if avg_quats_array_path is not None:
    filter.AvgQuatsArrayPath = avg_quats_array_path
  if statistics_array_name is not None:
    filter.StatisticsArrayName = statistics_array_name
  if include_radial_dist_func is not None:
    filter.IncludeRadialDistFunc = include_radial_dist_func
  if calculate_morphological_stats is not None:
    filter.CalculateMorphologicalStats = calculate_morphological_stats
  if size_distribution_fit_type is not None:
    filter.SizeDistributionFitType = size_distribution_fit_type
  if compute_aspect_ratio_distribution is not None:
    filter.ComputeAspectRatioDistribution = compute_aspect_ratio_distribution
  if aspect_ratio_distribution_fit_type is not None:
    filter.AspectRatioDistributionFitType = aspect_ratio_distribution_fit_type
  if compute_omega3_distribution is not None:
    filter.ComputeOmega3Distribution = compute_omega3_distribution
  if omega3_distribution_fit_type is not None:
    filter.Omega3DistributionFitType = omega3_distribution_fit_type
  if compute_neighborhood_distribution is not None:
    filter.ComputeNeighborhoodDistribution = compute_neighborhood_distribution
  if neighborhood_distribution_fit_type is not None:
    filter.NeighborhoodDistributionFitType = neighborhood_distribution_fit_type
  if calculate_crystallographic_stats is not None:
    filter.CalculateCrystallographicStats = calculate_crystallographic_stats
  if calculate_odf is not None:
    filter.CalculateODF = calculate_odf
  if calculate_mdf is not None:
    filter.CalculateMDF = calculate_mdf
  if calculate_axis_odf is not None:
    filter.CalculateAxisODF = calculate_axis_odf
  if size_correlation_resolution is not None:
    filter.SizeCorrelationResolution = size_correlation_resolution
  filter.execute()
  return filter.ErrorCode

